from ._bspline import bspline_curves, bspline_embeddings, uniform_augmented_knots
from ._legendre import legendre_curves
from ._normalization import clamp, rational

__all__ = ["clamp", "rational", "uniform_augmented_knots", "bspline_curves", "bspline_embeddings", "legendre_curves"]
